/*
 * This file is part of Distributed John,
 * Copyright (C) 2001-2003 by Luis Parravicini
 * http://ktulu.com.ar
 */

#ifndef __DEFS_H__
#define __DEFS_H__

#include "word.h"
#include <inttypes.h>

extern int getopt(int argc, char * const argv[], const char *optstring);
extern char *optarg;
extern int optind;

#define SERVER_PORT	40000

#define UNDEF_SLEEP	-1

/* possible actions for client/server commnucation */

/*  client requesting a packet of work */
#define GET_WORK	0

/*  GET_WORK answer */
#define FIRST_WORD	1

/*  sent by the client to notify the server a certain packet of work
    has been processed
*/
#define FINISH_WORK	2

/* sent by the client at startup to get the current charset */
#define GET_CHARSET	3

/* GET_CHARSET answer */
#define CHARSET_DATA	4

/* sent by the server if there's no available work, telling the client how much
 * to wait (in seconds) before asking again for more work
 */
#define NOW_SLEEP	5

/* sent by the client on startup to ask for the code of the external mode to
 * be used to crack work packets
 */
#define GET_CODE	6

/* server response to a GET_CODE packet
 */
#define CODE_DATA	7


/*
 * this struct is used to pass a pointer to any of the rest of the declared
 * structs to a function in a generic way.
 */
struct generic_packet {
	uint8_t		action	__attribute__ ((packed));
	uint32_t	size	__attribute__ ((packed));
};
#define GENERIC_PACKET_SIZE (sizeof(struct generic_packet))

struct getwork {
	uint8_t		action	__attribute__ ((packed));
	uint32_t	size	__attribute__ ((packed));
};
#define GETWORK_SIZE(x) (sizeof(struct getwork))

struct firstword {
	uint8_t		action	__attribute__ ((packed));
	uint32_t	size	__attribute__ ((packed));
	uint64_t	unit_id	__attribute__ ((packed));
	struct word	word1	__attribute__ ((packed));
	struct word	word2	__attribute__ ((packed));
};
#define FIRSTWORD_SIZE(x) (GENERIC_PACKET_SIZE + sizeof(uint64_t) + \
				WORD_SIZE(&((struct firstword*)x)->word1) + \
				WORD_SIZE(&((struct firstword*)x)->word2))

struct finishwork {
	uint8_t		action		__attribute__ ((packed));
	uint32_t	size		__attribute__ ((packed));
	uint64_t	unit_id		__attribute__ ((packed));
	uint8_t		fragment	__attribute__ ((packed));
	uint32_t	cracked		__attribute__ ((packed));	/* how many passwords were found     */
	uint32_t	length		__attribute__ ((packed));		/* length of all the passwords founds*/
	uint8_t		passwd[0]	__attribute__ ((packed));	/* passwords come after this struct  */
					/* as asciiz strings		     */
};
#define FINISHWORK_SIZE(x) (GENERIC_PACKET_SIZE + sizeof(uint64_t) + \
			sizeof(uint8_t) + sizeof(uint32_t) + sizeof(uint32_t) +\
			((struct finishwork*)x)->length * sizeof(uint8_t))

struct getcharset {
	uint8_t		action	__attribute__ ((packed));
	uint32_t	size	__attribute__ ((packed));
};
#define GETCHARSET_SIZE(x) (sizeof(struct getcharset))

struct charsetdata {
	uint8_t		action		__attribute__ ((packed));
	uint32_t	size		__attribute__ ((packed));
	uint8_t		length		__attribute__ ((packed));
	uint8_t		charset[256]	__attribute__ ((packed));
};
#define CHARSETDATA_SIZE(x) (GENERIC_PACKET_SIZE + sizeof(uint8_t) + \
				sizeof(uint8_t) * \
				((struct charsetdata*)x)->length)

struct sleep {
	uint8_t		action	__attribute__ ((packed));
	uint32_t	size	__attribute__ ((packed));
	uint64_t	secs	__attribute__ ((packed));
};
#define SLEEP_SIZE(x) (sizeof(struct sleep))

struct codedata {
	uint8_t		action	__attribute__ ((packed));
	uint32_t	size	__attribute__ ((packed));
	uint32_t	length	__attribute__ ((packed));
	uint8_t		data[16384]	__attribute__ ((packed)); /* this sucks but for now it works */
};
#define CODEDATA_SIZE(x) (GENERIC_PACKET_SIZE + sizeof(uint32_t) + \
			sizeof(uint8_t) * ((struct codedata*)x)->length)

struct getcode  {
	uint8_t		action	__attribute__ ((packed));
	uint32_t	size	__attribute__ ((packed));
};
#define GETCODE_SIZE(x) (GENERIC_PACKET_SIZE)

int packetsize(int, const struct generic_packet*);

#endif

